/*
 * sound functions
 */
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <multimedia/audio_hdr.h>
#include <multimedia/ulaw2linear.h>

#include "xplayer.h"
#include "xp_defs.h"

/*
 * Externs
 */

extern	void		dsp_msg();	/* xp_utils.c		*/
extern	void		start_play();	/* xp_utils.c		*/
extern	void		frm_busy();	/* xp_utils.c		*/
extern	void		xp_log();	/* xp_utils.c		*/
extern	void		speed();	/* xp_speed.c		*/


/*
 * Globals
 */

int	nb_loaded = 0;		/* nombre de sons en memoire	*/
int	nb_selected = 0;	/* nombre de sons a jouer	*/
Sau_desc	au[MAX_AU];	/* loaded sounds		*/
int	audio_fd = -1;		/* fd for audio i/o device	*/

/*
 * load a sound
 */

void		au_load()
{
  char		fullname[MAX_PATH+MAX_NAME];
  char		path[MAX_PATH];
  char		name[MAX_NAME];
  char		temp[120];
  unsigned char	*pdata;
  int		slot;
  int		fd;	/* file descriptor	*/
  int		valid;	/* is the file really a sound file?	*/
  int		open_error = FALSE;
  int		size;
  int		i, nbsamples;
  short		mag, curmag;
  int		inactive;
  Xv_opaque	set_snd;
  struct stat	st;
  
  frm_busy(Xplayer_pop_load->pop_load, TRUE);
  
  xv_set(Xplayer_win_xplayer->btn_log,
		PANEL_INACTIVE, FALSE,
		NULL);
  xv_set(Xplayer_pop_log->pop_log,
		XV_SHOW, TRUE,
		NULL);
		
  /* Set speed to zero, in case the file chooser was pinned	*/
  /*	instead of being called by the "load" button"		*/
  speed(0);
  
  /* Get filename and slot					*/
  sprintf(path, "%s", (char *) xv_get(Xplayer_pop_load->txt_lpath,
  					PANEL_VALUE));
  sprintf(name, "%s", (char *) xv_get(Xplayer_pop_load->txt_lname,
  					PANEL_VALUE));
  sprintf(fullname, "%s/%s", path, name);
  
  slot = (int) xv_get(Xplayer_pop_load->set_lslot, PANEL_VALUE);
  
  sprintf(temp, "\nLoading <%s> in slot %d\n", name, slot+1);
  xp_log(temp);
  
  /* Prepare the actual loading					*/
  
  if(au[slot].selected == E_NO_SOUND)	/* no sound in this slot*/
    nb_loaded++;
  else
  {
    if(au[slot].selected == E_ON)
      nb_selected -= 1;
      
    au[slot].selected = E_NO_SOUND;
    (void) free((char *) au[slot].data);
    /* no need to initialize the rest, it will be overwritten	*/
  }
  
  open_error = FALSE;
  
  if (((fd = open(fullname, O_RDONLY)) < 0) || (fstat(fd, &st) < 0)) 
  {
    /* File opening has failed					*/
    dsp_msg("File error");
    sprintf(temp, "*\tCan't read <%s>\n", fullname);
    xp_log(temp);
    open_error = TRUE;
  }
  else if(audio_read_filehdr(fd, &(au[slot].desc),
  			au[slot].info, MAX_INFO) != AUDIO_SUCCESS)
  {
    /* File has no sound header					*/
    dsp_msg("File error");
    sprintf(temp, "*\tThis file has not a valid sound header\n");
    xp_log(temp);
    open_error = TRUE;
  }
  else if(au[slot].desc.data_size == AUDIO_UNKNOWN_SIZE)
  {
    /* file size is not initialized				*/
    /*	determine the size					*/
    au[slot].desc.data_size = st.st_size - lseek(fd, 0L, L_INCR);
  }
  
  if(!open_error)
    if ((au[slot].desc.encoding != AUDIO_ENCODING_ULAW) ||
	    (au[slot].desc.bytes_per_unit != 1) ||
	    (au[slot].desc.samples_per_unit != 1) ||
	    (au[slot].desc.channels != 1))
    {
      sprintf(temp,
	    "*\tAudio encoding cannot be played properly\n");
      xp_log(temp);
      open_error = TRUE;
    }

  if(open_error)
  {
    nb_loaded--;
    xp_log("*\tFile error - Aborting load\n");
    sprintf(name, "No sound");
    inactive = TRUE;
  }
  else
  {    
    inactive = FALSE;
    /* load file						*/
    
    dsp_msg("Loading...");
    
    /* allocate a buffer, shrinking if request is too big	*/
    
    size = au[slot].desc.data_size;
    do 
    {
      au[slot].data = (unsigned char *)malloc(size);
    } while ((au[slot].data == NULL) && (size = size - (size / 8)));
    
    if(size != au[slot].desc.data_size)
    {
      /* Not enough memory to load the whole sound		*/
      sprintf(temp, "%.2f seconds of data truncated from '%s'.",
		    audio_bytes_to_secs(&au[slot].desc,
		    (size - au[slot].desc.data_size)), name);
      xp_log(temp);
    }
    au[slot].desc.data_size = size;
    
    /* Read in as much data as possible and close the file	*/
    
    au[slot].desc.data_size = read(fd,
	    (char *)au[slot].data, (int)au[slot].desc.data_size);
    (void) close(fd);
    
    /* determine the magnitude of the sound			*/
    nbsamples = au[slot].desc.data_size;
    for(i = 0, mag = 0, pdata = au[slot].data;
        	i < nbsamples;
        	i++, pdata++)
    {
      curmag = ABS(audio_u2s(*pdata));
      if(curmag > mag)
        mag = curmag;
    }
    au[slot].magnitude = audio_s2d(mag);
        
    xv_set(Xplayer_win_xplayer->btn_speed,
		PANEL_INACTIVE, FALSE,
		NULL);
    
    xv_set(Time_popup_pop_time->pop_time,
		XV_SHOW, TRUE,
		NULL);
		
    /* Display some stats of the sound file, now that it has	*/
    /* been sucessfully loaded					*/
    au[slot].selected = E_OFF;
    sprintf(temp, "\tInfo : %s\n", au[slot].info);
    xp_log(temp);
    au[slot].length = audio_bytes_to_secs(&au[slot].desc, 
    				au[slot].desc.data_size);
    sprintf(temp, "\tSize : %d octets\n", au[slot].desc.data_size);
    xp_log(temp);
    sprintf(temp, "\tLength : %f s\n", au[slot].length);
    xp_log(temp);
    sprintf(temp, "\tMagnitude : %f\n", au[slot].magnitude);
    xp_log(temp);
    dsp_msg("");
  }
  
  /* Display the file name on the control panel		*/

  switch(slot + 1)
  {
    case 1 :
      set_snd = Xplayer_win_xplayer->set_snd1;
      break;
    
    case 2 :
      set_snd = Xplayer_win_xplayer->set_snd2;
      break;
      
    case 3 :
      set_snd = Xplayer_win_xplayer->set_snd3;
      break;
      
    case 4 :
      set_snd = Xplayer_win_xplayer->set_snd4;
      break;
      
    case 5 :
      set_snd = Xplayer_win_xplayer->set_snd5;
      break;
    
    default :
      fputs("Wrong slot\n", stderr);
      quit();
  }
  xv_set(set_snd, PANEL_INACTIVE, inactive,
        	PANEL_CHOICE_STRING, 0, name,
        	PANEL_VALUE, 0,
        	NULL);
        	
  if(nb_selected == 0)
    stop_play();
  else
    start_play();
    
  frm_busy(Xplayer_pop_load->pop_load, FALSE);
}

/*
 * clear all the sound files from the memory
 */

void		au_clean()
{
  int	i;
  
  for(i = 0; i < MAX_AU; i++)
    if(au[i].selected != E_NO_SOUND)
    {
      au[i].selected = E_NO_SOUND;
      (void) free((char *) au[i].data);
    }
}

/*
 *  flush queued output and close the audio device
 */

void		au_stop()
{
  if(audio_fd >= 0)
  {
    (void) audio_flush(audio_fd);
    (void) close(audio_fd);
    audio_fd = -1;
  }
}

/*
 * select a sound for playing
 */
 
void	au_select(slot)
int	slot;
{
  int	value;
  
  au_stop();
  
  switch(slot)
  {
    case 1 :
      value = (int) xv_get(Xplayer_win_xplayer->set_snd1, PANEL_VALUE);
      break;
      
    case 2 :
      value = (int) xv_get(Xplayer_win_xplayer->set_snd2, PANEL_VALUE);
      break;
    
    case 3 :
      value = (int) xv_get(Xplayer_win_xplayer->set_snd3, PANEL_VALUE);
      break;
    
    case 4 :
      value = (int) xv_get(Xplayer_win_xplayer->set_snd4, PANEL_VALUE);
      break;
    
    case 5 :
      value = (int) xv_get(Xplayer_win_xplayer->set_snd5, PANEL_VALUE);
      break;
      
    default :
      fprintf(stderr, "Wrong slot (%d) in au_select\n", slot);
      quit();
  }
  value &= 01;
  if(value == 0)
  {
    au[slot-1].selected = E_OFF;
    nb_selected -= 1;
  }
  else
  {       
    au[slot-1].selected = E_ON;
    nb_selected += 1;
  }
  if(nb_selected == 0)
    stop_play();
  else
    start_play();
}

/*
 * play a sound
 */

void		au_play(pau)
Sau_desc	*pau;
{
  int		cnt, rtn, offset;
  
  cnt = (pau->desc).data_size;
  offset = 0;
  while(cnt > 0)
  {
    rtn = write(audio_fd, (char *) (pau->data + offset), cnt);
    if(rtn > 0)
    {
      cnt -= rtn;
      offset += rtn;
    }
    else
    {
      dsp_msg("Audio dev write error!");
      break;
    }
  }
}

/*
 * play all the currently active sounds
 */

void		au_playall()
{
  static int	i = 0;
  
  if(nb_selected == 0)
    return;
    
  /* open the audio device, if it is not already open		*/
  if(audio_fd < 0)
  {
    audio_fd = open(AUDIO_DEV, O_WRONLY | O_NDELAY);
    if(audio_fd < 0)
    {
      if(errno != EBUSY)
        dsp_msg("Audio dev open error!");
/*
      else
        device is busy, try to open it later
*/
      return;
    }
  }
  
  while(au[i].selected != E_ON)	/* find a selected sound	*/
  {
    i = (i + 1) % MAX_AU;
  }
  au_play(&au[i]);
  i = (i + 1) % MAX_AU;
  /*
  if(au[i].selected == E_ON)
    au_play(&au[i]);
  i = (i + 1) % MAX_AU;
  */
}

/*
 * display the sounds' characteristics
 */

void		au_stats()
{
  int		i, tot_size;
  double	tot_time;
  char		temp[80];
  
  if(nb_loaded == 0)
  {
    sprintf(temp, "\nNo sound loaded yet\n");
    xp_log(temp);
    xv_set(Xplayer_win_xplayer->btn_log,
		PANEL_INACTIVE, FALSE,
		NULL);
    xv_set(Xplayer_pop_log->pop_log, XV_SHOW, TRUE, NULL);
  }
  else if(nb_selected == 0)
  {
    sprintf(temp, "\nNo sound selected\n");
    xp_log(temp);
  }
  else
  {
    tot_size = 0;
    tot_time = 0;
  
    for(i = 0; i < MAX_AU; i++)
    if(au[i].selected == E_ON)
    {
      tot_size += au[i].desc.data_size;
      tot_time += au[i].length;
    }
    sprintf(temp, "\nSelected sounds :\n");
    xp_log(temp);
    sprintf(temp, "\tSpace used : %d bytes\n", tot_size);
    xp_log(temp);
    sprintf(temp, "\tLength: %f s\n", tot_time);
    xp_log(temp);
  }

}

/*
 * set the play volume (gain)
 */
 
void 		set_vol(vol)
int		vol;
{
  double	gain;
  
  /* open the audio device, if it is not already open		*/
  if(audio_fd < 0)
  {
    audio_fd = open(AUDIO_DEV, O_WRONLY | O_NDELAY);
    if(audio_fd < 0)
    {
      if(errno != EBUSY)
        dsp_msg("Audio dev open error!");
      /*
      else
        device is busy, try to open it later
      */
      return;
    }
  }
  
  gain = vol / 100.0;
  audio_set_play_gain(audio_fd, &gain);
}


/*
 * set the play volume (gain)
 */

void		au_volume()
{
  int		value;
  
  value = (int) xv_get(Time_popup_pop_time->sld_volume, PANEL_VALUE);
  
  set_vol(value);
  if(value == 0)
    speed(0);
}

/*
 * save a sound
 */

void		au_save()
{
  char		fullname[MAX_PATH+MAX_NAME];
  char		path[MAX_PATH];
  char		name[MAX_NAME];
  char		temp[120];
  int		slot;
  int		fd;	/* file descriptor	*/
  int		save_error = FALSE;
  int		size;
  struct stat	st;
  int		append = FALSE;
  int		choice;
  Audio_hdr	tmphdr;
    
  xv_set(Xplayer_pop_log->pop_log,
		XV_SHOW, TRUE,
		NULL);
		
  /* Set speed to zero, in case the file chooser was pinned	*/
  /*	instead of being called by the "save as..." button"	*/
  speed(0);
  
  /* Get filename and slot					*/
  sprintf(path, "%s", (char *) xv_get(Xplayer_pop_save->txt_spath,
  					PANEL_VALUE));
  sprintf(name, "%s", (char *) xv_get(Xplayer_pop_save->txt_sname,
  					PANEL_VALUE));
  sprintf(fullname, "%s/%s", path, name);
  
  slot = (int) xv_get(Xplayer_pop_save->set_sslot, PANEL_VALUE);

  /* update info string						*/
  sprintf(au[slot].info, "%s", 
  			xv_get(Xplayer_pop_save->txt_sinfo, PANEL_VALUE));
  			 
  sprintf(temp, "\nSaving sound #%d in <%s>\n", slot+1, fullname);  
  xp_log(temp);
  
  if(au[slot].selected == E_NO_SOUND)
  {
    sprintf(temp,"*\tNo sound to save in the selected slot\n");
    xp_log(temp);
    return;
  }
    
  if(stat(fullname, &st) == 0)
  {
    /* File already exists					*/
    choice = notice_prompt(Xplayer_pop_save->pop_save, NULL,
    	NOTICE_MESSAGE_STRINGS,
    		"The specified sound file",
    		fullname,
    		"already exists",
    		NULL,
    	NOTICE_BUTTON, "Cancel", 300,
    	NOTICE_BUTTON, "Append", 100,
    	NOTICE_BUTTON, "Replace", 200,
    	NULL);
     if(choice == 300)
       return;
     else if(choice == 100)
     {
       xp_log("Appending sound to existing file...");
       append = TRUE;
     }
  }
  
  
  if(append)
  {
    if((fd = open(fullname, O_RDWR)) < 0)
    {
      dsp_msg("File error");
      sprintf(temp, "*\tCan't append to <%s>\n", fullname);
      xp_log(temp);
      return;
    }
    
    /* make sure we are appending to an audio file		*/
    if(audio_read_filehdr(fd, &tmphdr, (char *) NULL, 0)
  			!= AUDIO_SUCCESS)
    {
      /* file has no sound header				*/
      dsp_msg("File error");
      sprintf(temp, "*\tThis file has not a valid sound header\n");
      xp_log(temp);
      return;
    }
    
    lseek(fd, st.st_size, L_SET);
  }
  else
  {
    /* create a new file, or replace an existing one		*/
    fd = open(fullname, O_WRONLY | O_CREAT | O_TRUNC, 0666);
    if((fd < 0) || 
    	(audio_write_filehdr(fd, &(au[slot].desc), au[slot].info,
    	 ((unsigned)strlen( au[slot].info))) != AUDIO_SUCCESS))
    {
      dsp_msg("File error");
      sprintf(temp, "*\tCan't create <%s>\n", fullname);
      xp_log(temp);
      if(fd >= 0)
        (void) close(fd);
      return;
    }
  }
  
  frm_busy(Xplayer_pop_save->pop_save, TRUE);
  
  if(write(fd, (char *) au[slot].data, au[slot].desc.data_size)
  		!= au[slot].desc.data_size)
  {
    dsp_msg("File error");
    sprintf(temp, "*\tError writing <%s>\n", fullname);
    xp_log(temp);
  }
  else if(append)
  {
    /* update the sound's size					*/
    audio_rewrite_filesize(fd, 
    		(tmphdr.data_size + au[slot].desc.data_size));
  }

  (void) close(fd);
  frm_busy(Xplayer_pop_save->pop_save, FALSE);
}


