/*
 * xp_utils.c
 * miscellaneous ui functions
 */

#include <xview/xview.h>
#include <xview/frame.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <string.h>
#include "xplayer.h"
#include "xp_defs.h" 
 
/*
 * externs
 */

extern	void	speed();	/* xp_speed.c			*/
extern	void	au_stop();	/* xp_sound.c			*/
extern	void	au_clean();	/* xp_sound.c			*/

extern	int	nb_loaded;	/* xp_sound.c			*/
extern	int	nb_selected;	/* xp_sound.c			*/


/* Display a (short) message in the left footer of the base	*/
/*	frame							*/

void	dsp_msg(msg)
char	*msg;
{
  xv_set(Xplayer_win_xplayer->win_xplayer,
 		 FRAME_LEFT_FOOTER, msg,
		 NULL);
}

/*
 * Add a string to the log window
 */

void	xp_log(str)
char	*str;
{
  textsw_insert(Xplayer_pop_log->tpn_log, str, strlen(str));
  textsw_possibly_normalize(Xplayer_pop_log->tpn_log,
      (Textsw_index) xv_get(Xplayer_pop_log->tpn_log, TEXTSW_INSERTION_POINT));
}

/* Stop playing							*/

void	stop_play()
{

  /* Select speed=0 and disable slider				*/
  speed(0); /* Tell notifier to stop updating the displays	*/

  xv_set(Time_popup_pop_time->sld_speed,
		PANEL_INACTIVE, TRUE,
		NULL);

  /* Disable settings						*/
  xv_set(Time_popup_pop_time->set_speed,
		PANEL_INACTIVE, TRUE,
		NULL);

  xv_set(Time_popup_pop_time->sld_volume,
		PANEL_INACTIVE, TRUE,
		NULL);

  if((nb_loaded > 0) && (nb_selected == 0))
    dsp_msg("Select a sound");
  else if(nb_loaded == 0)
    dsp_msg("Load a sound file");
}


/* Start (or restart) playing					*/

void	start_play()
{
  xv_set(Time_popup_pop_time->pop_time,
		XV_SHOW, TRUE,
		NULL);

  xv_set(Time_popup_pop_time->sld_volume,
		PANEL_INACTIVE, FALSE,
		NULL);

  /* Enable slider						*/
  xv_set(Time_popup_pop_time->sld_speed,
		PANEL_INACTIVE, FALSE,
		NULL);

  /* Enable settings						*/
  xv_set(Time_popup_pop_time->set_speed,
		PANEL_INACTIVE, FALSE,
		NULL);
}

/* Change the state of a frame : Busy <-> Not Busy		*/

void	frm_busy(frame, busy)
Frame	frame;
int	busy;
{
  if(busy == TRUE)
    xv_set(frame, FRAME_BUSY, TRUE, NULL);
  else
    xv_set(frame, FRAME_BUSY, FALSE, NULL);
}

/* Set the default position of the application's frames		*/

void	set_pos()
{
  Rect	base, time, file, log;

  frame_get_rect(Xplayer_win_xplayer->win_xplayer, &base);
  frame_get_rect(Time_popup_pop_time->pop_time, &time);
  frame_get_rect(Xplayer_pop_load->pop_load, &file);
  frame_get_rect(Xplayer_pop_log->pop_log, &log);

  base.r_left		= 10;
  base.r_top		= 10;
  time.r_left		= base.r_left + base.r_width + 15;
  time.r_top		= base.r_top;
  file.r_left	= base.r_left;
  file.r_top	= base.r_top + base.r_height + 30;
  log.r_left	= base.r_left;
  log.r_top	= file.r_top + file.r_height + 10;

  frame_set_rect(Xplayer_win_xplayer->win_xplayer, &base);
  frame_set_rect(Time_popup_pop_time->pop_time, &time);
  frame_set_rect(Xplayer_pop_load->pop_load, &file);
  frame_set_rect(Xplayer_pop_log->pop_log, &log);
}

/* Destroy all the UI objects before exiting the program	*/

void	quit()
{
  xv_destroy_safe(Xplayer_win_xplayer->win_xplayer);
}

/*
 * clean up everything before quitting
 */

Notify_value	destroy_func(client, status)
Notify_client	client;
Destroy_status	status;
{
  if(status == DESTROY_CLEANUP)
  {
    au_stop();
    au_clean();
    xv_set(Xplayer_pop_log->pop_log, XV_SHOW, FALSE, NULL);
    textsw_reset(Xplayer_pop_log->tpn_log, 0, 0);
    return notify_next_destroy_func(client, status);
  }
  return NOTIFY_DONE;
}
